package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICodeManager;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.product.CourseDTO;
import cn.rankin.common.utils.dto.product.SubRelationDTO;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.CourseSubTypeEnum;
import cn.rankin.common.utils.util.ListToMapUtil;
import cn.rankin.productservice.entity.*;
import cn.rankin.productservice.repository.*;
import cn.rankin.productservice.utils.DTOConverter;
import cn.rankin.productservice.vo.CourseSubItemVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class CourseService {

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private LessonRepository lessonRepository;

    @Autowired
    private CourseSubRelationRepository subRelationRepository;

    public boolean exists(String code) {
        Long count = courseRepository.countByCode(code);
        return count > 0L;
    }

    public boolean checkSubItemIds(List<SubRelationDTO> subRelationDTOList) {
        // TODO: 目前只包含课
        if (subRelationDTOList == null || subRelationDTOList.size() == 0) {
            return true;
        }

        List<String> lessonIdList = new ArrayList<>();
        subRelationDTOList.forEach( r -> {
            String subId = r.getId();
            CourseSubTypeEnum type = r.getType();
            if (type.equals(CourseSubTypeEnum.LESSON)) {
                lessonIdList.add(subId);
            }
        });
        Long count = lessonRepository.countByIdIn(lessonIdList);
        if (count < lessonIdList.size()) {
            return false;
        }
        return true;
    }

    @Transactional
    public List<CourseSubRelation> updateSubRelation(String courseId, List<SubRelationDTO> subRelationDTOList) {
        // TODO: 暂时不考虑单元
        if (subRelationDTOList == null) {
            return null;
        }

        Long count = subRelationRepository.deleteByCourseId(courseId);
        log.info("delete course sub item, code={}, num={}", courseId, String.valueOf(count));

        List<CourseSubRelation> subRelationList = new ArrayList<>();
        for (int i = 0; i < subRelationDTOList.size(); i++) {
            SubRelationDTO relationDTO = subRelationDTOList.get(i);
            CourseSubRelation relation = new CourseSubRelation();
            relation.setCourseId(courseId);
            relation.setSubId(relationDTO.getId());
            relation.setType(relationDTO.getType());
            relation.setSort(i);
            subRelationList.add(relation);
        }

        List<CourseSubRelation> result = subRelationRepository.save(subRelationList);
        return result;
    }

    public APIResult<Page<Course>> search(Course course, Integer pageNo, Integer pageSize) {
        Long count = courseRepository.count(course);
        Page<Course> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Course> wareList = courseRepository.find(course, page.getStart(), pageSize);
        page.setList(wareList);

        return APIResult.ok(page);
    }

    @Transactional
    public APIResult<Course> create(CourseDTO courseDTO) {
        String code = courseDTO.getCode();
        if (exists(code)) {
            return APIResult.error(APICodeManager.ALREADY_EXISTS);
        }

        // sub item check
        List<SubRelationDTO> subRelationDTOList = courseDTO.getSubList();
        if (!checkSubItemIds(subRelationDTOList)) {
            return APIResult.error(APICodeManager.error("包含错误课编码"));
        }


        Course course = DTOConverter.convert(courseDTO);
        Course result = courseRepository.save(course);

        String courseId = result.getId();
        updateSubRelation(courseId, subRelationDTOList);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Course> update(CourseDTO courseDTO) {
        List<SubRelationDTO> subRelationDTOList = courseDTO.getSubList();
        if (!checkSubItemIds(subRelationDTOList)) {
            return APIResult.error(APICodeManager.error("包含错误课编码"));
        }

        Course course = DTOConverter.convert(courseDTO);
        Course result = courseRepository.save(course);

        String courseId = result.getId();
        updateSubRelation(courseId, subRelationDTOList);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = courseRepository.updateStatusById(id, BaseStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(APICodeManager.NOT_EXISTS);
    }

    public APIResult<Course> findById(String id) {
        //TODO: 目前只加载课, 不加载单元
        Course course = courseRepository.findById(id);
        if (course == null) {
            return APIResult.error(APICodeManager.NOT_EXISTS);
        }

        // 你瞅瞅, 加上单元难不难?
        List<CourseSubRelation> subRelationList = subRelationRepository.findByCourseId(id);
        if (subRelationList == null || subRelationList.size() == 0) {
            return APIResult.ok(course);
        }

        List<CourseSubItemVo> subItemVoList = new ArrayList<>();

        List<String> lessonIdList = new ArrayList<>();
        subRelationList.forEach( rel -> {
            CourseSubTypeEnum type = rel.getType();
            String subId = rel.getSubId();
            if (type.equals(CourseSubTypeEnum.LESSON)) {
                lessonIdList.add(subId);
            }
        });

        List<Lesson> lessonList = lessonRepository.findByIds(lessonIdList);
        Map<String, Lesson> lessonMap = new ListToMapUtil<String, Lesson>(lessonList).parse("id");

        for(CourseSubRelation rel: subRelationList) {
            String subCode = rel.getSubId();
            CourseSubTypeEnum type = rel.getType();
            if (type.equals(CourseSubTypeEnum.LESSON)) {
                Lesson lesson = lessonMap.get(subCode);
                CourseSubItemVo vo = CourseSubItemVo.parseToVo(lesson);
                subItemVoList.add(vo);
            }
        }

        course.setSubItemList(subItemVoList);

        return APIResult.ok(course);
    }

}
