package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICodeManager;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.product.ItemDTO;
import cn.rankin.common.utils.enums.ItemStatusEnum;
import cn.rankin.productservice.entity.Item;
import cn.rankin.productservice.repository.*;
import cn.rankin.productservice.utils.DTOConverter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;

@Slf4j
@Service
public class ItemService {

    @Autowired
    private ItemRepository itemRepository;

    @Autowired
    private CourseRepository courseRepository;

    public boolean checkCourseCode(String code) {
        Long count = courseRepository.countByCode(code);
        return count > 0L;
    }

    public boolean exists(Item item) {
        Long count = itemRepository.count(item);
        return count > 0L;
    }

    public APIResult<Page<Item>> search(Item item, Integer pageNo, Integer pageSize) {
        Long count = itemRepository.count(item);
        Page<Item> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Item> itemList = itemRepository.find(item, page.getStart(), pageSize);
        page.setList(itemList);

        return APIResult.ok(page);
    }

    @Transactional
    public APIResult<Item> create(ItemDTO itemDTO) {
        // 检查course是否存在
        String code = itemDTO.getCode();
        if (!checkCourseCode(code)) {
            return APIResult.error(APICodeManager.error("课程不存在"));
        }

        // 检查course与merchantId的唯一性
        Item sample = new Item();
        sample.setCourseId(itemDTO.getCourseId());
        sample.setMerchantId(itemDTO.getMerchantId());
        if (exists(sample)) {
            return APIResult.error(APICodeManager.ALREADY_EXISTS);
        }

        // 保存item数据
        Item item = DTOConverter.convert(itemDTO);
        Item result = itemRepository.save(item);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String itemId) {
        Integer count = itemRepository.updateStatusById(itemId, ItemStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(APICodeManager.NOT_EXISTS);
    }

    @Transactional
    public APIResult<Item> update(ItemDTO itemDTO) {
        // 检查course是否存在
        String code = itemDTO.getCode();
        if (!checkCourseCode(code)) {
            return APIResult.error(APICodeManager.error("课程不存在"));
        }

        // 检查course与merchantId是否存在
        Item sample = new Item();
        sample.setCourseId(itemDTO.getCourseId());
        sample.setMerchantId(itemDTO.getMerchantId());
        if (!exists(sample)) {
            return APIResult.error(APICodeManager.ALREADY_EXISTS);
        }

        // 保存item数据
        Item item = DTOConverter.convert(itemDTO);
        Item result = itemRepository.update(item);

        return APIResult.ok(result);
    }
}
