package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICodeManager;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.product.SupportDTO;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.productservice.entity.Support;
import cn.rankin.productservice.entity.SupportSelfRelation;
import cn.rankin.productservice.repository.SupportRepository;
import cn.rankin.productservice.repository.SupportSelfRelationRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Slf4j
@Service
public class SupportService {

    @Autowired
    private SupportRepository supportRepository;

    @Autowired
    private SupportSelfRelationRepository selfRelationRepository;

    public static Support convert(SupportDTO supportDTO) {
        Support support = new Support();
        BeanUtils.copyProperties(supportDTO, support);
        Set<String> supportIdSet = supportDTO.getSupportList();
        if (supportIdSet != null && supportIdSet.size() > 0) {
            Set<Support> supportSet = new HashSet<>();
            supportIdSet.forEach( e -> {
                Support s = new Support();
                s.setId(e);
                supportSet.add(s);
            });
            support.setSupportList(supportSet);
        }
        return support;
    }

    public APIResult<Page<Support>> search(Support support, Integer pageNo, Integer pageSize) {
        Long count = supportRepository.count(support);
        Page<Support> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Support> supportList = supportRepository.find(support, page.getStart(), pageSize);
        page.setList(supportList);

        return APIResult.ok(page);
    }

    public boolean exists(String code) {
        Long count = supportRepository.countByCode(code);
        return count > 0;
    }

    public boolean checkSupportIds(Set<String> supportSet) {
        if (supportSet == null || supportSet.size() == 0) {
            return true;
        }

        List<String> supportIdList = new ArrayList<>();
        supportIdList.addAll(supportSet);
        Long count = supportRepository.countByIdIn(supportIdList);
        if (count < supportSet.size()) {
            return false;
        }
        return true;
    }

    @Transactional
    public List<SupportSelfRelation> updateSupportList(String supportId, Set<String> supportIdSet) {
        if (supportIdSet == null) {
            return null;
        }

        selfRelationRepository.deleteAllByFid(supportId);

        List<SupportSelfRelation> supportSelfRelationList = new ArrayList<>();
        for (String s : supportIdSet) {
            SupportSelfRelation relation = new SupportSelfRelation();
            relation.setFid(supportId);
            relation.setTid(s);
            supportSelfRelationList.add(relation);
        }

        return selfRelationRepository.save(supportSelfRelationList);
    }

    @Transactional
    public APIResult<Support> create(SupportDTO supportDTO) {
        String code = supportDTO.getCode();
        if (exists(code)) {
            return APIResult.error(APICodeManager.ALREADY_EXISTS);
        }

        Set<String> supportIdSet = supportDTO.getSupportList();
        if (!checkSupportIds(supportIdSet)) {
            return APIResult.error(APICodeManager.error("包含错误的id"));
        }

        Support support = convert(supportDTO);
        Support result = supportRepository.save(support);

        String supportId = result.getId();
        updateSupportList(supportId, supportIdSet);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Support> update(SupportDTO supportDTO) {
        Set<String> supportIdSet = supportDTO.getSupportList();
        if (!checkSupportIds(supportIdSet)) {
            return APIResult.error(APICodeManager.error("存在错误的周边编号"));
        }

        Support support = convert(supportDTO);
        Support result = supportRepository.save(support);
        
        String supportId = result.getId();
        updateSupportList(supportId, supportIdSet);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = supportRepository.updateStatusById(id, BaseStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }

        return APIResult.error(APICodeManager.NOT_EXISTS);
    }

    public APIResult<Support> findById(String id) {
//        SupportVo supportVo = new SupportVo();
        Support support = supportRepository.find(id);
        if (support == null) {
            return APIResult.error(APICodeManager.NOT_EXISTS);
        }
//        }else {
//            BeanUtils.copyProperties(support, supportVo);
//        }

//        List<String> supportIdList = selfRelationRepository.findRefByFid(id);
//        List<Support> supportList = supportRepository.findByIds(supportIdList);
//
//        supportVo.setSupportList(supportList);

        return APIResult.ok(support);
    }

}
